package org.pfsw.bif;

import static org.junit.Assert.*;
import static org.pfsw.bif.BasicUtil.*;

import java.math.BigInteger;
import java.math.RoundingMode;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.Vector;
import java.util.concurrent.atomic.AtomicBoolean;

import org.junit.Test;
import org.pfsw.bif.execution.IExecutable;
import org.pfsw.bif.http.StandardHttpHeaderName;
import org.pfsw.bif.testhelper.LargeInteger;
import org.pfsw.bif.testhelper.SampleEnum1;
import org.pfsw.bif.testhelper.UnitTestHelper;

public class BasicUtilTest
{
  @Test
  public void test_noop()
  {
    BasicUtil.noop();
  }

  @Test
  public void test_isNoneNull_false()
  {
    Object[] strings = { "alpha", "beta", null, "gamma" };
    assertFalse(BasicUtil.current().isNoneNull(strings));
  }

  @Test
  public void test_isNoneNull_true()
  {
    Object[] strings = { "alpha", "beta", "gamma" };
    assertTrue(BU.isNoneNull(strings));
  }

  @Test
  public void test_isAnyNull_false()
  {
    Object[] strings = { "alpha", "beta", "gamma" };
    assertFalse(BU.isAnyNull(strings));
  }

  @Test
  public void test_isAnyNull_true()
  {
    Object[] strings = { "alpha", "beta", "gamma", null, "delta", null };
    assertTrue(BU.isAnyNull(strings));
  }

  @Test
  public void test_isNullOrEmpty_Array_1()
  {
    Integer[] array = { new Integer(8), new Integer(32), new Integer(69) };

    assertTrue(!BU.isNullOrEmpty(array));
  }

  @Test
  public void test_isNullOrEmpty_Array_2()
  {
    Integer[] array = null;

    assertTrue(BU.isNullOrEmpty(array));
  }

  @Test
  public void test_isNullOrEmpty_Array_3()
  {
    Integer[] array = {};

    assertTrue(BU.isNullOrEmpty(array));
  }

  @Test
  public void test_isNullOrEmpty_Collection_1()
  {
    List<Object> coll = new ArrayList<Object>();

    coll.add("Test");
    coll.add(new Integer(8));
    assertTrue(!BU.isNullOrEmpty(coll));
  }

  @Test
  public void test_isNullOrEmpty_Collection_2()
  {
    Set<Object> coll = new HashSet<Object>();

    assertTrue(BU.isNullOrEmpty(coll));
  }

  @Test
  public void test_isNullOrEmpty_Collection_3()
  {
    Vector<Object> coll = null;

    assertTrue(BU.isNullOrEmpty(coll));
  }

  @Test
  public void test_isNullOrEmpty_Map_1()
  {
    Map<Object, Object> map = new HashMap<Object, Object>();

    map.put("Test", new Date());
    map.put(new Integer(11), "eleven");
    assertTrue(!BU.isNullOrEmpty(map));
  }

  @Test
  public void test_isNullOrEmpty_Map_2()
  {
    @SuppressWarnings("rawtypes")
    Map map = new HashMap();

    assertTrue(BU.isNullOrEmpty(map));
  }

  @Test
  public void test_isNullOrEmpty_Map_3()
  {
    Map<Object, Object> map = null;

    assertTrue(BU.isNullOrEmpty(map));
  }

  @Test
  public void test_isAnyNullOrBlank_null()
  {
    assertTrue(BU.isAnyNullOrBlank((String)null));
  }

  @Test
  public void test_isAnyNullOrBlank_null_array()
  {
    assertTrue(BU.isAnyNullOrBlank((String[])null));
  }

  @Test
  public void test_isAnyNullOrBlank_empty()
  {
    assertFalse(BU.isAnyNullOrBlank());
  }

  @Test
  public void test_isAnyNullOrBlank_true_one()
  {
    assertTrue(BU.isAnyNullOrBlank("\n  \b"));
  }

  @Test
  public void test_isAnyNullOrBlank_true_many()
  {
    assertTrue(BU.isAnyNullOrBlank("Peter", "Paul ", "\n  \b", "Fred"));
  }

  @Test
  public void test_isAnyNullOrBlank_true_many_null()
  {
    assertTrue(BU.isAnyNullOrBlank("Peter", "Mary", "Paul ", null, "Fred"));
  }

  @Test
  public void test_isAnyNullOrBlank_false_one()
  {
    assertFalse(BU.isAnyNullOrBlank("Peter"));
  }

  @Test
  public void test_isAnyNullOrBlank_false_many()
  {
    assertFalse(BU.isAnyNullOrBlank("Peter", "Mary", "Paul ", " Joe", "Fred"));
  }

  @Test
  public void test_isNoneNullOrBlank_null()
  {
    assertFalse(BU.isNoneNullOrBlank((String)null));
  }

  @Test
  public void test_isNoneNullOrBlank_empty()
  {
    assertTrue(BU.isNoneNullOrBlank());
  }

  @Test
  public void test_isNoneNullOrBlank_false_one()
  {
    assertFalse(BU.isNoneNullOrBlank("\n  \b"));
  }

  @Test
  public void test_isNoneNullOrBlank_false_many()
  {
    assertFalse(BU.isNoneNullOrBlank("Peter", "Paul ", "\n  \b", "Fred"));
  }

  @Test
  public void test_isNoneNullOrBlank_false_many_null()
  {
    assertFalse(BU.isNoneNullOrBlank("Peter", "Mary", "Paul ", null, "Fred"));
  }

  @Test
  public void test_isNoneNullOrBlank_true_one()
  {
    assertTrue(BU.isNoneNullOrBlank("Peter"));
  }

  @Test
  public void test_isNoneNullOrBlank_true_many()
  {
    assertTrue(BU.isNoneNullOrBlank("Peter", "Mary", "Paul ", " Joe", "Fred"));
  }

  @Test
  public void test_isNullOrEmpty_1()
  {
    String[] strings = null;
    assertTrue(BU.isNullOrEmpty(strings));
  }

  @Test
  public void test_isNullOrEmpty_2()
  {
    String[] strings = new String[0];
    assertTrue(BU.isNullOrEmpty(strings));
  }

  @Test
  public void test_isNullOrEmpty_3()
  {
    String[] strings = { "Bob" };
    assertTrue(!BU.isNullOrEmpty(strings));
  }

  @Test
  public void test_isNullOrBlank_1()
  {
    assertTrue(BU.isNullOrBlank(""));
  }

  @Test
  public void test_isNullOrBlank_2()
  {
    assertTrue(BU.isNullOrBlank("   "));
  }

  @Test
  public void test_isNullOrBlank_3()
  {
    assertTrue(BU.isNullOrBlank(null));
  }

  @Test
  public void test_isNullOrBlank_4()
  {
    assertTrue(BU.isNullOrBlank(" \t\t  "));
  }

  @Test
  public void test_isNullOrBlank_5()
  {
    assertTrue(!BU.isNullOrBlank(" X"));
  }

  @Test
  public void test_notNullOrEmpty_null()
  {
    assertFalse(BU.notNullOrEmpty((String)null));
  }

  @Test
  public void test_notNullOrEmpty_empty()
  {
    assertFalse(BU.notNullOrEmpty(""));
  }

  @Test
  public void test_notNullOrEmpty_filled()
  {
    assertTrue(BU.notNullOrEmpty("Something"));
  }

  @Test
  public void test_notNullOrBlank_null()
  {
    assertFalse(BU.notNullOrBlank((String)null));
  }

  @Test
  public void test_notNullOrBlank_empty()
  {
    assertFalse(BU.notNullOrBlank(""));
  }

  @Test
  public void test_notNullOrBlank_whitespaces()
  {
    assertFalse(BU.notNullOrBlank(" \t"));
  }

  @Test
  public void test_notNullOrBlank_filled()
  {
    assertTrue(BU.notNullOrBlank(" Something "));
  }

  @Test
  public void test_containsWhitespaceChars__false()
  {
    assertFalse(BU.containsWhitespaceChars(null));
    assertFalse(BU.containsWhitespaceChars(""));
    assertFalse(BU.containsWhitespaceChars("A"));
    assertFalse(BU.containsWhitespaceChars("Test.it-with_special$chars&!"));
  }

  @Test
  public void test_containsWhitespaceChars__True()
  {
    assertTrue(BU.containsWhitespaceChars(" "));
    assertTrue(BU.containsWhitespaceChars("Bad Ass"));
    assertTrue(BU.containsWhitespaceChars("A "));
    assertTrue(BU.containsWhitespaceChars(" A "));
    assertTrue(BU.containsWhitespaceChars("A\tB"));
    assertTrue(BU.containsWhitespaceChars("A\nB"));
    assertTrue(BU.containsWhitespaceChars("A\rB"));
    assertTrue(BU.containsWhitespaceChars("A\fB"));
  }

  @Test
  public void test_isLess__true()
  {
    assertTrue(BU.isLess("alpha", "beta"));
    assertTrue(BU.isLess(Integer.valueOf(7), Integer.valueOf(9)));
  }

  @Test
  public void test_isLess__false()
  {
    assertFalse(BU.isLess("delta", "beta"));
    assertFalse(BU.isLess(Integer.valueOf(9), Integer.valueOf(9)));
    assertFalse(BU.isLess(Integer.valueOf(9), Integer.valueOf(8)));
  }

  @Test
  public void test_isEqual__true()
  {
    assertTrue(BU.isEqual("alpha", "alpha"));
    assertTrue(BU.isEqual(Integer.valueOf(7), Integer.valueOf(7)));
  }

  @Test
  public void test_isEqual__false()
  {
    assertFalse(BU.isEqual("delta", "beta"));
    assertFalse(BU.isEqual(Integer.valueOf(5), Integer.valueOf(9)));
    assertFalse(BU.isEqual(Integer.valueOf(9), Integer.valueOf(8)));
  }

  @Test
  public void test_isGreater__true()
  {
    assertTrue(BU.isGreater("gamma", "beta"));
    assertTrue(BU.isGreater(Integer.valueOf(14), Integer.valueOf(9)));
  }

  @Test
  public void test_isGreater__false()
  {
    assertFalse(BU.isGreater("alpha", "beta"));
    assertFalse(BU.isGreater(Integer.valueOf(9), Integer.valueOf(9)));
    assertFalse(BU.isGreater(Integer.valueOf(7), Integer.valueOf(8)));
  }

  @Test
  public void test_asString__null()
  {
    assertNull(BU.asString(null));
  }

  @Test
  public void test_asString___standard_objects()
  {
    assertEquals("7", BU.asString(Integer.valueOf(7)));
    assertEquals("true", BU.asString(Boolean.TRUE));
  }

  @Test
  public void test_asString___IStringRepresentation()
  {
    assertEquals("alpha", BU.asString(SampleEnum1.ALPHA));
    assertEquals("Access-Control-Allow-Headers", BU.asString(StandardHttpHeaderName.ACCESS_CONTROL_ALLOW_HEADERS));
  }

  @Test
  public void test_asString___enum_value()
  {
    assertEquals("CEILING", BU.asString(RoundingMode.CEILING));
  }

  @Test
  public void test_isTypeOf__null()
  {
    assertFalse(BU.isTypeOf(null, String.class));
    assertFalse(BU.isTypeOf("test", null));
  }

  @SuppressWarnings("cast")
  @Test
  public void test_isTypeOf__inheritance()
  {
    Integer num1 = Integer.valueOf(6);
    Long num2 = Long.valueOf(99L);

    assertTrue(num1 instanceof Number);
    assertFalse(BU.isTypeOf(num1, Number.class));

    assertTrue(num2 instanceof Number);
    assertFalse(BU.isTypeOf(num2, Number.class));
  }

  @Test
  public void test_isTypeOf__true()
  {
    assertTrue(BU.isTypeOf(Integer.valueOf(6), Integer.class));
    assertTrue(BU.isTypeOf(Long.valueOf(6), Long.class));
    assertTrue(BU.isTypeOf("test", String.class));
  }

  @Test
  public void test_isTypeOf__false()
  {
    assertFalse(BU.isTypeOf(null, Long.class));
    assertFalse(BU.isTypeOf(Integer.valueOf(6), null));
    assertFalse(BU.isTypeOf(Integer.valueOf(6), Long.class));
    assertFalse(BU.isTypeOf(Long.valueOf(6), Integer.class));
    assertFalse(BU.isTypeOf("test", Integer.class));
    assertTrue(BU.isTypeOrSubTypeOf(new BigInteger("7"), BigInteger.class));
    assertTrue(BU.isTypeOrSubTypeOf(LargeInteger.valueOf("7"), BigInteger.class));
    assertTrue(BU.isTypeOrSubTypeOf(LargeInteger.valueOf("7"), Number.class));
  }

  @Test
  public void test_isTypeOrSubTypeOf__false()
  {
    assertFalse(BU.isTypeOrSubTypeOf(null, Long.class));
    assertFalse(BU.isTypeOrSubTypeOf(Integer.valueOf(6), null));
    assertFalse(BU.isTypeOrSubTypeOf(Integer.valueOf(6), Long.class));
    assertFalse(BU.isTypeOrSubTypeOf(Long.valueOf(6), Integer.class));
    assertFalse(BU.isTypeOrSubTypeOf(Long.valueOf(6), Integer.class));
    assertFalse(BU.isTypeOrSubTypeOf(new BigInteger("7"), LargeInteger.class));
    assertFalse(BU.isTypeOrSubTypeOf("test", Integer.class));
  }

  @Test
  public void test_isTypeOrSubTypeOf__true()
  {
    assertTrue(BU.isTypeOrSubTypeOf(Integer.valueOf(6), Integer.class));
    assertTrue(BU.isTypeOrSubTypeOf(Integer.valueOf(6), Number.class));
    assertTrue(BU.isTypeOrSubTypeOf(Long.valueOf(6), Number.class));
  }

  @Test
  public void test_firstValidValue__double()
  {
    Double value = 123.4d;

    assertTrue(value == BU.firstValidValue(Double.NaN, Double.NaN, value, 99.1d));
  }

  @Test
  public void test_firstValidValue__double_not_found()
  {
    assertNull(BU.firstValidValue(new Double[0]));
    assertNull(BU.firstValidValue(Double.NaN, Double.NaN));
  }

  @Test
  public void test_firstValidValue__float()
  {
    Float value = 123.88f;

    assertTrue(value == BU.firstValidValue(Float.NaN, Float.NaN, value, 837.35f));
  }

  @Test
  public void test_firstValidValue__float_not_found()
  {
    assertNull(BU.firstValidValue(new Float[0]));
    assertNull(BU.firstValidValue(Float.NaN, Float.NaN));
  }

  @Test
  public void test_asExecutable()
  {
    AtomicBoolean result = new AtomicBoolean();
    Runnable runnable = UnitTestHelper.createRunnableOn(result);
    IExecutable executable = BU.asExecutable(runnable);

    assertNull(BU.asExecutable(null));

    executable.execute();
    assertTrue(result.get());
  }

  @Test
  public void test_asRunnable()
  {
    AtomicBoolean result = new AtomicBoolean();
    IExecutable executable = UnitTestHelper.createExecutableOn(result);
    Runnable runnable = BU.asRunnable(executable);

    assertNull(BU.asRunnable(null));

    runnable.run();
    assertTrue(result.get());
  }
}