package org.pfsw.bif.conversion;

import static org.junit.Assert.*;

import org.junit.Test;

public class IBidirectionalConverterTest
{
  @Test
  public void test_to_internal()
  {
    assertEquals(Long.valueOf(26L), this.createConverter1().toInternal("26"));
  }
  
  @Test
  public void test_to_internal_null()
  {
    assertNull(this.createConverter1().toInternal(null));
  }
  
  @Test(expected=ConversionRuntimeException.class)
  public void test_to_internal_exception()
  {
    this.createConverter1().toInternal("no-number");
  }
  
  @Test
  public void test_to_external()
  {
    assertEquals("724", this.createConverter1().toExternal(Long.valueOf(724L)));
  }
  
  @Test
  public void test_to_external_null()
  {
    assertNull(this.createConverter1().toExternal(null));
  }
  
  // ===== Helper methods =====
  protected IBidirectionalConverter<Long, String> createConverter1() 
  {
    return new IBidirectionalConverter<Long, String>()
    {
      @Override
      public Long toInternal(String string) throws ConversionRuntimeException
      {
        if (string == null)
        {
          return null;
        }
        try
        {
          return Long.parseLong(string);
        }
        catch (NumberFormatException ex)
        {
          throw new ConversionRuntimeException(ex);
        }
      }
      
      @Override
      public String toExternal(Long value) throws ConversionRuntimeException
      {
        if (value == null)
        {
          return null;
        }
        return value.toString();
      }
    };
  }
}