// ===========================================================================
// CONTENT  : CLASS RemoteCallExceptionTest
// AUTHOR   : Manfred Duchrow
// VERSION  : 1.0 - 19/04/2021
// HISTORY  :
//  19/04/2021  mdu  CREATED
//
// Copyright (c) 2021, by MDCS. All rights reserved.
// ===========================================================================
package org.pfsw.bif.exceptions;

import static org.junit.Assert.*;

import java.io.IOException;

import org.junit.Test;

public class RemoteCallExceptionTest
{
  @Test
  public void test_put_and_get()
  {
    try
    {
      //@formatter:off
      throw RemoteCallException.create("test %d", 1)
          .put(null, "v1000")
          .put("k1", null)
          .put("k2", "v2")
      ;
      //@formatter:on
    }
    catch (RemoteCallException e)
    {
      assertEquals("test 1", e.getMessage());
      assertNull(e.get("k0"));
      assertNull(e.get("k1"));
      assertNull(e.get(null));
      assertEquals("v2", e.get("k2"));
    }
  }

  @Test
  public void test_with_cause()
  {
    try
    {
      throw RemoteCallException.create(new IOException("broken"), "test %d", 2);
    }
    catch (RemoteCallException e)
    {
      assertEquals("test 2", e.getMessage());
      assertTrue(e.getCause() instanceof IOException);
    }
  }
  
  @Test
  public void test_toString()
  {
    try
    {
      throw RemoteCallException.create(403, "test %d", 3);
    }
    catch (RemoteCallException e)
    {
      assertEquals("test 3", e.getMessage());
      assertEquals(403, e.getStatusCode());
      assertEquals("RemoteCallException(403, 'test 3')", e.toString());
    }
  }
}
