// ===========================================================================
// CONTENT  : TEST CLASS HttpMethodTest
// AUTHOR   : Manfred Duchrow
// VERSION  : 1.0 - 19/08/2014
// HISTORY  :
//  19/08/2014  mdu  CREATED
//
// Copyright (c) 2014, by MDCS. All rights reserved.
// ===========================================================================
package org.pfsw.bif.http;

import static org.junit.Assert.*;

import org.junit.Test;

public class HttpMethodTest
{
  @Test
  public void test_equals_hashCode()
  {
    HttpMethod getMethod = HttpMethod.GET;

    assertFalse(HttpMethod.PUT.equals(null));
    assertFalse(HttpMethod.CONNECT.equals("CONNECT"));
    assertTrue(HttpMethod.GET.equals(getMethod));
    assertTrue(HttpMethod.GET.hashCode() == getMethod.hashCode());
    assertFalse(HttpMethod.GET.equals(HttpMethod.DELETE));
    assertFalse(HttpMethod.GET.hashCode() == HttpMethod.POST.hashCode());
  }

  @Test
  public void test_getHttp11Method_not_found()
  {
    assertNull(HttpMethod.getHttp11Method("unknown"));
  }

  @Test
  public void test_getHttp11Method_GET()
  {
    assertSame(HttpMethod.GET, HttpMethod.getHttp11Method("get"));
  }

  @Test
  public void test_getHttp11Method_POST()
  {
    assertSame(HttpMethod.POST, HttpMethod.getHttp11Method("Post"));
  }

  @Test
  public void test_getHttp11Method_DELETE()
  {
    assertSame(HttpMethod.DELETE, HttpMethod.getHttp11Method("DELETE"));
  }

  @Test
  public void test_getHttp11Method_PUT()
  {
    assertSame(HttpMethod.PUT, HttpMethod.getHttp11Method("pUt"));
  }

  @Test
  public void test_getName_TRACE()
  {
    assertEquals("TRACE", HttpMethod.TRACE.getName());
  }

  @Test
  public void test_hasBody_POST()
  {
    assertTrue(HttpMethod.POST.hasBody());
  }

  @Test
  public void test_hasBody_PUT()
  {
    assertTrue(HttpMethod.PUT.hasBody());
  }

  @Test
  public void test_hasBody_DELETE()
  {
    assertFalse(HttpMethod.DELETE.hasBody());
  }

  @Test
  public void test_hasBody_HEAD()
  {
    assertFalse(HttpMethod.HEAD.hasBody());
  }

  @Test
  public void test_hasBody_GET()
  {
    assertFalse(HttpMethod.GET.hasBody());
  }
  
  @Test
  public void test_asString()
  {
    assertEquals("GET", HttpMethod.GET.asString());
    assertEquals("POST", HttpMethod.POST.asString());
    assertEquals("PATCH", HttpMethod.PATCH.asString());
    assertEquals("OPTIONS", HttpMethod.OPTIONS.asString());
    assertEquals("TRACE", HttpMethod.TRACE.asString());
  }
  
  @Test
  public void test_toString()
  {
    assertEquals("HttpMethod('GET')", HttpMethod.GET.toString());
    assertEquals("HttpMethod('PUT')", HttpMethod.PUT.toString());
    assertEquals("HttpMethod('DELETE')", HttpMethod.DELETE.toString());
    assertEquals("HttpMethod('CONNECT')", HttpMethod.CONNECT.toString());
  }
}
