// ===========================================================================
// CONTENT  : TEST CLASS ICharsetsTest
// AUTHOR   : Manfred Duchrow
// VERSION  : 1.0 - 20/03/2016
// HISTORY  :
//  20/03/2016  mdu  CREATED
//
// Copyright (c) 2016, by MDCS. All rights reserved.
// ===========================================================================
package org.pfsw.bif.text;

// ===========================================================================
// IMPORTS
// ===========================================================================
import static org.junit.Assert.*;
import java.nio.charset.Charset;

import org.junit.Test;

public class ICharsetsTest
{
  @Test
  public void test_constants()
  {
    String asciiText = "The quick brown fox jumped over the lazy dog. !$%&/()=?*#@_:;,.-\"+-~";
    String isoText = "Türschloßöffner kosten £30.-";
    String text = "Türschloßöffner kosten €30.-";

    assertEquals(asciiText, this.doubleConvert(asciiText, ICharsets.US_ASCII));
    assertEquals(text, this.doubleConvert(text, ICharsets.UTF_8));
    assertEquals(text, this.doubleConvert(text, ICharsets.UTF_16));
    assertEquals(text, this.doubleConvert(text, ICharsets.UTF_16LE));
    assertEquals(text, this.doubleConvert(text, ICharsets.UTF_16BE));
    assertEquals(isoText, this.doubleConvert(isoText, ICharsets.ISO_8859_1));
    assertEquals("Special ÜÖÄß ŠŤŁ", this.doubleConvert("Special ÜÖÄß ŠŤŁ", ICharsets.ISO_8859_2));
    assertEquals(isoText, this.doubleConvert(isoText, ICharsets.ISO_8859_3));
    assertEquals("Special ÜÖÄß ŠŖĢ", this.doubleConvert("Special ÜÖÄß ŠŖĢ", ICharsets.ISO_8859_4));
    assertEquals("Special ДЖЋ", this.doubleConvert("Special ДЖЋ", ICharsets.ISO_8859_5));
    assertEquals("Special ÜÖÄß ØÆŖ", this.doubleConvert("Special ÜÖÄß ØÆŖ", ICharsets.ISO_8859_13));
    assertEquals(text, this.doubleConvert(text, ICharsets.ISO_8859_15));
    assertEquals(text, this.doubleConvert(text, ICharsets.JVM_DEFAULT));
  }

  // =========================================================================
  // HELPER METHODS
  // =========================================================================
  protected String doubleConvert(String string, Charset charset)
  {
    return new String(string.getBytes(charset), charset);
  }
}