package org.pfsw.bif.text;

import static org.junit.Assert.*;

import org.junit.Test;

public class ObjectReferenceTest
{
  @Test
  public void test_asString()
  {
    ObjectReference ref1 = ObjectReference.create(Integer.valueOf(17));

    assertEquals("java.lang.Integer#17", ref1.asString());
  }

  @Test
  public void test_toString()
  {
    ObjectReference ref1 = ObjectReference.create(Long.valueOf(42));

    assertEquals("ObjectReference('java.lang.Long#42')", ref1.toString());
    assertEquals("ObjectReference('java.lang.Void#0')", ObjectReference.create(null).toString());
  }

  @Test
  public void test_equals_hashCode()
  {
    ObjectReference ref1 = ObjectReference.create(Long.valueOf(42));
    ObjectReference ref2 = ObjectReference.create(Integer.valueOf(42));
    ObjectReference ref3 = ObjectReference.create("test");
    ObjectReference ref4 = ObjectReference.create("test");
    ObjectReference ref5 = ObjectReference.create(null);
    ObjectReference ref6 = ObjectReference.create(Integer.valueOf(42));
    ObjectReference ref7 = ObjectReference.create(Integer.valueOf(43));

    assertFalse(ref1.equals(null));
    assertFalse(ref1.equals(Long.valueOf(42)));

    assertTrue(ref1.hashCode() != ref2.hashCode());
    assertFalse(ref1.equals(ref2));

    assertTrue(ref1.hashCode() != ref3.hashCode());
    assertFalse(ref1.equals(ref3));

    assertTrue(ref1.hashCode() != ref4.hashCode());
    assertFalse(ref1.equals(ref4));

    assertTrue(ref1.hashCode() != ref5.hashCode());
    assertFalse(ref1.equals(ref5));

    assertTrue(ref1.hashCode() != ref6.hashCode());
    assertFalse(ref1.equals(ref6));

    assertTrue(ref1.hashCode() != ref7.hashCode());
    assertFalse(ref1.equals(ref7));

    assertTrue(ref3.hashCode() == ref4.hashCode());
    assertTrue(ref3.equals(ref4));

    assertTrue(ref2.hashCode() == ref6.hashCode());
    assertTrue(ref2.equals(ref6));

    assertTrue(ref2.hashCode() != ref7.hashCode());
    assertFalse(ref2.equals(ref7));
  }

  @Test
  public void test_matches()
  {
    ObjectReference ref1 = ObjectReference.create(Integer.valueOf(42));
    ObjectReference ref2 = ObjectReference.create("test");

    assertTrue(ref1.matches(42));
    assertFalse(ref1.matches(45));
    assertTrue(ref2.matches("test"));
    assertFalse(ref2.matches("Test"));
    assertFalse(ref2.matches(null));
  }
}
